@@ -1,4 +1,5 @@
 """Module containing helper functions related to eumdac configuration files"""
+
 import os
 from pathlib import Path
 
@@ -11,3 +12,13 @@ def get_config_dir() -> Path:
 def get_credentials_path() -> Path:
     """get the Path to the credentials of eumdac"""
     return get_config_dir() / "credentials"
+
+
+def get_url_path() -> Path:
+    """get the Path to the tailor configurations of eumdac"""
+    r = get_config_dir() / "url"
+    r.mkdir(parents=True, exist_ok=True)
+    return r
+
+
+PERCENTAGE_WARNING = 90
