@@ -1,22 +1,23 @@
-"""Module contianing the data tailor class"""
+"""Module containing the Data Tailor class and related errors"""
+
 from __future__ import annotations
 
 import json
 from typing import TYPE_CHECKING
 
-import requests
-
 from eumdac.customisation import Customisation
 from eumdac.errors import EumdacError, eumdac_raise_for_status
 from eumdac.tailor_models import Chain, DataTailorCRUD, Filter, Quicklook, RegionOfInterest
+from eumdac.request import get, post
 import eumdac.common
+from eumdac.token import AccessToken, AnonymousAccessToken
 
 if TYPE_CHECKING:  # pragma: no cover
     import sys
     from typing import Any, Optional
 
     from eumdac.product import Product
-    from eumdac.token import AccessToken, URLs
+    from eumdac.token import BaseToken, URLs
 
     if sys.version_info < (3, 9):
         from typing import Iterable, Mapping, Sequence
@@ -25,7 +26,14 @@ if TYPE_CHECKING:  # pragma: no cover
 
 
 class DataTailor:
-    token: AccessToken
+    """Interface with the EUMETSAT Data Tailor Webservice
+
+    Instance it by providing a token and access the Data Tailor functions like
+    posting new customisation jobs, listing the current jobs, cancelling running ones,
+    download job outputs, and delete finished jobs.
+    """
+
+    token: BaseToken
     urls: URLs
     chains: DataTailorCRUD
     filters: DataTailorCRUD
@@ -34,7 +42,7 @@ class DataTailor:
     _info: Optional[Mapping[str, Any]] = None
     _user_info: Optional[Mapping[str, Any]] = None
 
-    def __init__(self, token: AccessToken) -> None:
+    def __init__(self, token: BaseToken) -> None:
         self.token = token
         self.urls = token.urls
         self.chains = DataTailorCRUD(self, Chain)
@@ -44,8 +52,9 @@ class DataTailor:
 
     @property
     def customisations(self) -> Sequence[Customisation]:
+        """Return the list of customisations"""
         url = self.urls.get("tailor", "customisations")
-        response = requests.get(
+        response = get(
             url,
             auth=self.token.auth,
             headers=eumdac.common.headers,
@@ -56,10 +65,11 @@ class DataTailor:
 
     @property
     def info(self) -> Mapping[str, Any]:
+        """Return information about Data Tailor Webservice in a Dict-like format."""
         if self._info is None:
             url = self.urls.get("tailor", "info")
             auth = self.token.auth
-            response = requests.get(
+            response = get(
                 url,
                 auth=auth,
                 headers=eumdac.common.headers,
@@ -70,10 +80,11 @@ class DataTailor:
 
     @property
     def user_info(self) -> Mapping[str, Any]:
+        """Return information about the current Data Tailor Webservice user in a Dict-like format."""
         if self._user_info is None:
             url = self.urls.get("tailor", "user info")
             auth = self.token.auth
-            response = requests.get(
+            response = get(
                 url,
                 auth=auth,
                 headers=eumdac.common.headers,
@@ -84,9 +95,10 @@ class DataTailor:
 
     @property
     def quota(self) -> Mapping[str, Any]:
+        """Return information about the user workspace quota on the Data Tailor Webservice in a Dict-like format."""
         url = self.urls.get("tailor", "report quota")
         auth = self.token.auth
-        response = requests.get(
+        response = get(
             url,
             auth=auth,
             headers=eumdac.common.headers,
@@ -94,16 +106,47 @@ class DataTailor:
         eumdac_raise_for_status("Could not get quota", response, DataTailorError)
         return response.json()
 
+    @property
+    def is_local(self) -> bool:
+        """Return if the configured Data Tailor is the Data Tailor Webservice or a local instance."""
+        # when no token for datatailor exists we assume this is a local tailor instance
+        return isinstance(self.token, AnonymousAccessToken)
+
     def get_customisation(self, cutomisation_id: str) -> Customisation:
+        """Return a customisation job given its id"""
         return Customisation(cutomisation_id, self)
 
     def new_customisation(self, product: Product, chain: Chain) -> Customisation:
+        """Start a new customisation job for the given product.
+
+        Started customisations will run asynchronously and need to be monitored.
+        Once finished, their outputs can be downloaded, and then they need to be deleted.
+
+        Arguments
+        ---------
+        - `product` : *Product*
+            Data Store product to customise
+        - `chain` : *Chain*
+            Chain configuration to use for the customisation
+        """
         (customisation,) = self.new_customisations([product], chain)
         return customisation
 
     def new_customisations(
         self, products: Iterable[Product], chain: Chain
     ) -> Sequence[Customisation]:
+        """Starts multiple customisation jobs for the given products
+
+        Started customisations will run asynchronously and need to be monitored.
+        Once finished, their outputs can be downloaded, and then they need to be deleted.
+
+        Arguments
+        ---------
+        - `products` : *Iterable[Product]*
+            Data Store products to customise
+        - `chain` : *Chain*
+            Chain configuration to use for the customisation
+        """
         product_paths = "|||".join(
             self.urls.get(
                 "datastore",
@@ -115,17 +158,30 @@ class DataTailor:
             )
             for product in products
         )
-        params = {"product_paths": product_paths, "access_token": str(self.token)}
+
+        data = {"product_paths": product_paths}
+        params = {}
+
+        # instead of guessing the correct token, datatailor should use the token attached to each product
+        if isinstance(self.token, AccessToken):
+            # provide own token to the endpoint since we assume it is valid for datastore
+            params["access_token"] = str(self.token)
+        elif self.is_local and any(products):
+            # for local tailor instances we use the token attached to the first product
+            params["access_token"] = str(next(iter(products)).datastore.token)
         if isinstance(chain, str):
-            params["chain_name"] = chain
+            data["chain_name"] = chain
         else:
-            params["chain_config"] = json.dumps(chain.asdict())
-        response = requests.post(
+            data["chain_config"] = json.dumps(chain.asdict())
+
+        response = post(
             self.urls.get("tailor", "customisations"),
             auth=self.token.auth,
             params=params,
+            files=data,
             headers=eumdac.common.headers,
         )
+
         eumdac_raise_for_status("Could not add customizations", response, DataTailorError)
 
         customisation_ids = response.json()["data"]
