@@ -1,5 +1,6 @@
 """module containing the TailorApp which will be used when using 
 eumdac download **with** the --tailor argument."""
+
 from __future__ import annotations
 
 import concurrent
@@ -11,17 +12,21 @@ import threading
 import time
 import typing
 from collections import namedtuple
-from concurrent.futures import Future
 from datetime import timedelta
 from pathlib import Path
-from typing import *
+from typing import Any, Callable, Dict, Generator, Iterable, List, Optional, Tuple
 
-from eumdac.customisation import Customisation, CustomisationError, UnableToGetCustomisationError
+from eumdac.config import PERCENTAGE_WARNING
+from eumdac.customisation import (
+    Customisation,
+    CustomisationError,
+    UnableToGetCustomisationError,
+)
 from eumdac.datastore import DataStore
 from eumdac.datatailor import DataTailor
 from eumdac.errors import EumdacError
-from eumdac.fake import FakeDataStore, FakeDataTailor  # type: ignore
-from eumdac.job_id import JobIdentifactor
+from eumdac.futures import EumdacFutureFunc, EumdacThreadPoolExecutor
+from eumdac.job_id import JobIdentifier
 from eumdac.logging import logger
 from eumdac.order import Order
 from eumdac.product import Product
@@ -35,38 +40,12 @@ class FailedStateTransitionError(Exception):
 
     def find_lines(self, search_string: str) -> Generator[str, None, None]:
         if self.faillog:
-            for l in self.faillog.splitlines():
-                if search_string in l:
-                    result_text = l.split(" - ")[-1]
+            for line in self.faillog.splitlines():
+                if search_string in line:
+                    result_text = line.split(" - ")[-1]
                     yield result_text
 
 
-class TailorFutureFunc:
-    def __init__(self) -> None:
-        self.aborted = False
-
-    def __call__(self, *args: Any, **kwargs: Any) -> None:
-        raise NotImplementedError()
-
-    def abort(self) -> None:
-        self.aborted = True
-
-
-class TailorThreadPoolExecutor(concurrent.futures.ThreadPoolExecutor):
-    def __init__(self, *args: Any, **kwargs: Any) -> None:
-        self.functors: List[TailorFutureFunc] = []
-        super().__init__(*args, **kwargs)
-
-    def pool_shutdown(self) -> None:
-        for f in self.functors:
-            f.abort()
-        return super().shutdown()
-
-    def pool_submit(self, fn: TailorFutureFunc, *args: Any, **kwargs: Any) -> Future[Any]:
-        self.functors.append(fn)
-        return super().submit(fn, *args, **kwargs)
-
-
 class TailorApp:
     def __init__(
         self,
@@ -74,8 +53,8 @@ class TailorApp:
         datastore: Any,
         datatailor: Any,
     ) -> None:
-        self.server_state_monitor_executor = TailorThreadPoolExecutor(max_workers=3)
-        self.download_executor = TailorThreadPoolExecutor(max_workers=2)
+        self.server_state_monitor_executor = EumdacThreadPoolExecutor(max_workers=3)
+        self.download_executor = EumdacThreadPoolExecutor(max_workers=2)
         self.datastore = datastore
         self.datatailor = datatailor
         self.order = order
@@ -85,13 +64,15 @@ class TailorApp:
             self.output_dir.mkdir(exist_ok=True, parents=True)
             self.chain = Chain(**order_d["chain"])
 
-    def run(self) -> None:
+    def run(self) -> bool:
         try:
             return self.resume()
         except FatalEumdacError as fee:
             logger.error(f"Fatal error during execution: {fee}")
             self.order.remote_delete_failed(self.datatailor)
 
+        return False
+
     def shutdown(self) -> None:
         self.server_state_monitor_executor.pool_shutdown()
         self.download_executor.pool_shutdown()
@@ -105,30 +86,37 @@ class TailorApp:
         customization_add_func: Callable[[Product], Customisation],
         timeout: float = 600,
     ) -> None:
-        customisation_futures: List[Future[Any]] = []
+        customisation_futures: List[concurrent.futures.Future[Any]] = []
         download_futures = []
 
         failed_customisations = []
         done_customistations = []
 
         num_jobs = len(list(self.order.iter_product_info()))
-        job_identificator = JobIdentifactor(num_jobs)
+        job_identificator = JobIdentifier(num_jobs)
 
         for product in products:
             customisation_futures.append(
                 self.server_state_monitor_executor.pool_submit(
                     WaitForDoneCustomisationFutureFunc(
-                        customization_add_func, self.order, job_identificator
+                        customization_add_func,
+                        self.order,
+                        job_identificator,
                     ),
                     product,
                 )
             )
 
         while customisation_futures:
-            done_concurrent_futures: List[Future[Any]] = []
-            not_done_concurrent_futures: List[Future[Any]] = customisation_futures
+            done_concurrent_futures: List[concurrent.futures.Future[Any]] = []
+            not_done_concurrent_futures: List[
+                concurrent.futures.Future[Any]
+            ] = customisation_futures
             try:
-                (done_concurrent_futures, not_done_concurrent_futures,) = [
+                (
+                    done_concurrent_futures,
+                    not_done_concurrent_futures,
+                ) = [
                     list(x)
                     for x in concurrent.futures.wait(
                         customisation_futures,
@@ -165,6 +153,9 @@ class TailorApp:
                     logger.error(f"{completed_customistation} timed out: {te}")
                     failed_customisations.append(completed_customistation)
                     continue
+                except CustomisationError as ce:
+                    logger.error(f"Failed: {ce}")
+                    continue
                 except Exception as exc:
                     logger.error(f"{done_future} failed: {exc}")
                     failed_customisations.append(done_future)
@@ -185,7 +176,7 @@ class TailorApp:
 
         _ = concurrent.futures.wait(download_futures, return_when=concurrent.futures.ALL_COMPLETED)
 
-    def resume(self) -> None:
+    def resume(self) -> bool:
         # handle existing customisations
         customisations = []
         products_to_resume = []
@@ -194,6 +185,18 @@ class TailorApp:
         # query all customisation states
         success_customisation_product_futures = []
 
+        try:
+            user_name = self.datatailor.user_info["username"]
+            quota_info = self.datatailor.quota["data"][user_name]
+            if quota_info["space_usage_percentage"] > PERCENTAGE_WARNING:
+                logger.warning(f"Reaching maximum quota: {quota_info['space_usage_percentage']}%")
+            elif quota_info["space_usage_percentage"] > 100:
+                logger.warning(f"Over maximum quota: {quota_info['space_usage_percentage']}%")
+        except EumdacError as e:
+            # The quota call is unsupported by local-tailor, so we don't report
+            if not self.datatailor.is_local:
+                logger.warning(f"Could not determine current quota: {e}")
+
         for p_info in self.order.iter_product_info():
             success_customisation_product_futures.append(
                 self.server_state_monitor_executor.pool_submit(
@@ -234,8 +237,10 @@ class TailorApp:
                 customization_add_func=lambda x: self.datatailor.new_customisation(x, self.chain),
             )
 
+        return True
 
-class DownloadRunFutureFunc(TailorFutureFunc):
+
+class DownloadRunFutureFunc(EumdacFutureFunc):
     def __init__(self, order: Order, output_dir: Path, job_id: str):
         super().__init__()
         self.order = order
@@ -258,7 +263,7 @@ class DownloadRunFutureFunc(TailorFutureFunc):
                 filtered_results.extend(matches)
             results = filtered_results
 
-        logger.debug(f"{self.job_id} Starting download(s) for {results}")
+        logger.debug(f"{self.job_id}: Starting download(s) for {results}")
         download_states = {result: "PENDING" for result in results}
         self.order.update(
             customisation._id,
@@ -267,6 +272,9 @@ class DownloadRunFutureFunc(TailorFutureFunc):
             download_states=download_states,
         )
 
+        with self.order.dict_from_file() as order_d:
+            dirs = order_d["dirs"]
+
         num_total, num = self.order.resolve_product_num(product._id)
         for result in results:
             if self.aborted:
@@ -274,18 +282,18 @@ class DownloadRunFutureFunc(TailorFutureFunc):
             download_states[result] = "DOWNLOAD_ERROR"
             try:
                 logger.info(
-                    f"{self.job_id} Downloading output of job {customisation._id} for {product._id}"
+                    f"{self.job_id}: Downloading output of job {customisation._id} for {product._id}"
                 )
-                self.download_customisation_result(customisation, result)
-                logger.info(f"{self.job_id} {Path(result).parts[-1]} has been downloaded.")
+                self.download_customisation_result(customisation, result, dirs)
+                logger.info(f"{self.job_id}: {Path(result).parts[-1]} has been downloaded.")
                 download_states[result] = "DOWNLOADED"
             except DownloadExistsError as err:
-                logger.info(f"{self.job_id} Skipping download. File exists: {err.product_path}")
+                logger.info(f"{self.job_id}: Skipping download. File exists: {err.product_path}")
                 download_states[result] = "DOWNLOADED"
             except DownloadAbortedError as err:
-                logger.warning(f"{self.job_id} Download of {err.product_path} aborted")
+                logger.warning(f"{self.job_id}: Download of {err.product_path} aborted")
             except Exception as exc:
-                logger.error(f"{self.job_id} Error while downloading: {exc}")
+                logger.error(f"{self.job_id}: Error while downloading: {exc}")
 
             self.order.update(
                 customisation._id,
@@ -297,22 +305,35 @@ class DownloadRunFutureFunc(TailorFutureFunc):
         # delete serverside customisation on success
         if "DOWNLOAD_ERROR" not in download_states.values():
             logger.info(
-                f"{self.job_id} Deleting customization {customisation._id} for {product._id}"
+                f"{self.job_id}: Deleting customization {customisation._id} for {product._id}"
             )
             customisation.delete()
         else:
-            logger.warning(f"{self.job_id} {customisation} download failed. Keeping customisation.")
+            logger.warning(
+                f"{self.job_id}: {customisation} download failed. Keeping customisation."
+            )
 
     fatal_error_logs = {
         "ERROR": ["invalid", "INTERNAL ERROR"],
     }
 
-    def download_customisation_result(self, customisation: Customisation, result: str) -> None:
+    def download_customisation_result(
+        self, customisation: Customisation, result: str, dirs: bool
+    ) -> None:
         product_path = self.output_dir / Path(result).parts[-1]
+
+        if dirs:
+            # when the dirs flag is used
+            # a subdirectory is created
+            # to avoid overwriting common files
+            output_subdir = self.output_dir / f"{Path(result).parts[0]}"
+            output_subdir.mkdir(exist_ok=True)
+            product_path = output_subdir / Path(result).parts[-1]
+
         if product_path.is_file():
             raise DownloadExistsError(product_path)
 
-        with tempfile.TemporaryDirectory() as tempdir:
+        with tempfile.TemporaryDirectory(dir=self.output_dir, suffix=".tmp") as tempdir:
             tmp_prod_p = Path(tempdir) / product_path.parts[-1]
             with tmp_prod_p.open("wb") as tmp_prod:
                 with customisation.stream_output_iter_content(result) as chunks:
@@ -325,7 +346,7 @@ class DownloadRunFutureFunc(TailorFutureFunc):
 
 def check_invalid_state_transition_changelog(error: FailedStateTransitionError) -> None:
     fatal_error_logs = {
-        "ERROR": ["invalid", "INTERNAL ERROR", "incompatible"],
+        "ERROR": ["INTERNAL ERROR", "incompatible"],
     }
     # find messages containing errors
     for severity, filters in fatal_error_logs.items():
@@ -335,7 +356,7 @@ def check_invalid_state_transition_changelog(error: FailedStateTransitionError)
                 raise FatalEumdacError(EumdacError(line, desc))
 
 
-class StateQueryFutureFunc(TailorFutureFunc):
+class StateQueryFutureFunc(EumdacFutureFunc):
     @typing.no_type_check
     def __call__(
         self,
@@ -356,7 +377,10 @@ class StateQueryFutureFunc(TailorFutureFunc):
             _ = customisation.status
             success = True
         except (CustomisationError, CustomisationTimeoutError) as ce:
-            download_states = p_info["download_states"]
+            download_states = {}
+            if "download_states" in p_info:
+                download_states = p_info["download_states"]
+
             if "DOWNLOADED" in download_states.values():
                 logger.warning(
                     f"Customisation {customisation_id} has already been finished and downloaded."
@@ -389,13 +413,13 @@ class GetCustomisation:
 State = namedtuple("State", "name log")
 
 
-class WaitForDoneCustomisationFutureFunc(TailorFutureFunc):
+class WaitForDoneCustomisationFutureFunc(EumdacFutureFunc):
     def __init__(
         self,
         customization_add_func: Callable[[Product], Customisation],
         order: Order,
-        job_identificator: JobIdentifactor,
-        timeout: float = 600,
+        job_identificator: JobIdentifier,
+        timeout: float = 1800,
         polling_interval: float = 1.0,
         max_retries: int = 10,
         max_timeouts: int = 3,
@@ -439,9 +463,9 @@ class WaitForDoneCustomisationFutureFunc(TailorFutureFunc):
         self.job_identificator.register(product)
         self.job_id = self.job_identificator.job_id_str(product)
 
-        logger.info(
-            f"Job {self.job_identificator.registered_objects[product]} of "
-            f"{self.job_identificator.total_jobs} starting."
+        logger.info(f"Triggering {self.job_id.lower()} of {self.job_identificator.total_jobs}")
+        logger.debug(
+            f"{self.job_id} tracked as {self.job_identificator.registered_objects[product]}"
         )
         customisation = self.try_to_add_customisation(product)
         return self.wait_for_success(customisation, product)
@@ -454,7 +478,7 @@ class WaitForDoneCustomisationFutureFunc(TailorFutureFunc):
 
             if retries >= self.max_retries:
                 raise CustomisationError(
-                    f"{self.job_id} Could not add customisation after {retries} retries"
+                    f"{self.job_id}: Could not add customisation after {retries} retries"
                 )
             try:
                 return self.customization_add_func(product)
@@ -462,11 +486,11 @@ class WaitForDoneCustomisationFutureFunc(TailorFutureFunc):
                 retries += 1
                 try:
                     check_error_response(e)
-                except ExceedingNumberOfCustomisationsEumdacError as exceeding_number_error:
-                    self.order.set_num_workers(exceeding_number_error.number_of_customisations)
+                except ExceedingNumberOfCustomisationsEumdacError as _exceeding_number_error:
+                    pass
 
                 logger.warning(
-                    f"{self.job_id} {e}: Could not create customisation. Retry: {retries}"
+                    f"{self.job_id}: {e}: Could not create customisation. Retry: {retries}"
                 )
                 time.sleep(self.wait_timedelta.total_seconds())
 
@@ -483,15 +507,16 @@ class WaitForDoneCustomisationFutureFunc(TailorFutureFunc):
                 while not self.aborted and not self.terminated:
                     self.step(customisation, product._id)
                     if self.timed_out:
-                        logger.error(f"{self.job_id} {self} timed_out")
+                        logger.error(f"{self.job_id}: {self} timed_out")
                         retries += 1
                         timeouts += 1
                         if timeouts >= self.max_timeouts:
                             raise FatalEumdacError(
-                                EumdacError(f"{self.job_id} {customisation._id} timed out")
+                                EumdacError(f"{self.job_id}: {customisation._id} timed out")
                             )
                 break
             except FailedStateTransitionError as fste:
+                logger.debug(f"{fste}: {fste.faillog}")
                 check_invalid_state_transition_changelog(fste)
 
                 if customisation:
@@ -501,7 +526,7 @@ class WaitForDoneCustomisationFutureFunc(TailorFutureFunc):
 
                 retries += 1
                 logger.debug(
-                    f"{self.job_id} {self} failed with {fste} on try {retries}/{self.max_retries}"
+                    f"{self.job_id}: {self} failed with {fste} on try {retries}/{self.max_retries}"
                 )
                 if retries < self.max_retries:
                     # if we retry this reinitialize state
@@ -518,7 +543,7 @@ class WaitForDoneCustomisationFutureFunc(TailorFutureFunc):
                 self.state_timer.cancel()
                 if retries >= self.max_retries:
                     raise FatalEumdacError(
-                        EumdacError(f"{self.job_id} {customisation._id} is inactive")
+                        EumdacError(f"{self.job_id}: {customisation._id} is inactive")
                     )
 
         return customisation, product
@@ -531,14 +556,18 @@ class WaitForDoneCustomisationFutureFunc(TailorFutureFunc):
         if self.terminated:
             return
         if self.failed:
+            logs = self.get_latest_log(customisation)
+            if not self.order._no_warning_logs:
+                log_message = logs.split("\n\n")
+                logger.warning(f"{self.job_id}: {log_message[-1]}")
             raise FailedStateTransitionError(
-                f"{self.job_id} Server state is: {self.state.name}",
-                faillog=self.get_latest_log(customisation),
+                f"{self.job_id}: Server state is: {self.state.name}",
+                faillog=logs,
             )
 
         if self.state.name not in self.state_transitions.keys():
             raise InvalidStateTransitionError(
-                "{self.job_id} Unexpected State: {self.state.name})",
+                "{self.job_id}: Unexpected State: {self.state.name})",
                 old_state=self.state,
                 new_state=State(self.state.name, self.get_latest_log(customisation)),
             )
@@ -551,7 +580,7 @@ class WaitForDoneCustomisationFutureFunc(TailorFutureFunc):
         new_state = customisation.status
         if self.state.name != new_state:
             logger.debug(
-                f"{self.job_id} {customisation}: State change: {self.state.name} -> {new_state}"
+                f"{self.job_id}: {customisation}: State change: {self.state.name} -> {new_state}"
             )
             num_total, num = self.order.resolve_product_num(product_id)
             suffix = {
@@ -562,12 +591,13 @@ class WaitForDoneCustomisationFutureFunc(TailorFutureFunc):
                 "INACTIVE": "is inactive",
             }
             logger.info(
-                f"{self.job_id} Customisation {customisation._id} for {product_id} {suffix[new_state]}"
+                f"{self.job_id}: Customisation {customisation._id} for {product_id} {suffix[new_state]}"
             )
+
         if new_state not in self.state_transitions[self.state.name]:
             self.state_timer.cancel()
             raise InvalidStateTransitionError(
-                f"{self.job_id} Tried to transition from state {self.state} to {new_state}, "
+                f"{self.job_id}: Tried to transition from state {self.state} to {new_state}, "
                 "which is not expected",
                 old_state=self.state,
                 new_state=State(new_state, self.get_latest_log(customisation)),
@@ -584,13 +614,13 @@ class WaitForDoneCustomisationFutureFunc(TailorFutureFunc):
                 )
 
     def windup_timer(self) -> threading.Timer:
-        logger.debug(f"{self.job_id} {repr(self)} windup_timer")
+        logger.debug(f"{self.job_id}: {repr(self)} windup_timer")
         timer = threading.Timer(self.timeout, self.on_timeout)
         timer.start()
         return timer
 
     def on_timeout(self) -> None:
-        logger.debug(f"{self.job_id} {repr(self)} on_timeout")
+        logger.debug(f"{self.job_id}: {repr(self)} on_timeout")
         self.timed_out = True
 
     def get_latest_log(self, customisation: Customisation) -> str:
@@ -657,14 +687,22 @@ def check_error_response(error: Exception) -> None:
     }
 
     if isinstance(error, EumdacError):
-        if error.extra_info and error.extra_info["status"] in fatal_error_responses:
+        if (
+            error.extra_info
+            and error.extra_info["status"] in fatal_error_responses
+            and "description" in error.extra_info
+        ):
             if any(
                 fatal_response in error.extra_info["description"]
                 for fatal_response in fatal_error_responses[error.extra_info["status"]]
             ):
                 raise FatalEumdacError(error)
 
-        if error.extra_info and error.extra_info["status"] in number_of_customisation_responses:
+        if (
+            error.extra_info
+            and error.extra_info["status"] in number_of_customisation_responses
+            and "description" in error.extra_info
+        ):
             if any(
                 number_response in error.extra_info["description"]
                 for number_response in number_of_customisation_responses[error.extra_info["status"]]
